// This function handles the Paging for Event items
function PageEvents(selectFirstPage) {
	var eventsContainer = $('.events-container');
	var eventsPerPage = 6; // Change this value to the number of item you want to display in each page
	
	// When building the paging mechanism, only the currently visible event items are taken into consideration
	// as the hidden once are hidden due to filtering. Everytime a filter changes, the paging is rebuilt.
	var _events = eventsContainer.find('.event-item:visible');
	
	var pages = [[]];
	var currentPage = 0;
	var currentEvent = 0;
	
	// Keep creating pages and "attaching" events to each page until you run out of events
	while(_events.length > currentEvent) {
		var eventsInCurrentPage = _events.slice(currentEvent, currentEvent + eventsPerPage);
		pages[currentPage] = [];
		$.each(eventsInCurrentPage, function() {
			pages[currentPage].push($(this).attr('id'));
		});
		
		currentPage++;
		currentEvent = currentEvent + eventsPerPage;
	}
	
	var pagingContainer = eventsContainer.find('.events-paging ul');
	
	// If the Paging element has not yet been added to the DOM, create it.
	if(pagingContainer.length <= 0) {
		eventsContainer.append("<div style='display: none' class='col-xs-12 events-paging'><ul></ul></div>");
		pagingContainer = eventsContainer.find('.events-paging ul');
	}

	pagingContainer.html(''); // Empty the paging container
	
	// Render the page numbers and assign them an onClick function
	$.each(pages, function(index, value) {
		var pageNum = index + 1;
		
		pagingContainer.append("<li id='page-" + pageNum + "' events='" + pages[index].toString() + "'>" + pageNum + "</li>");
		
		var pageItem = pagingContainer.find('li#page-' + pageNum);
		
		// If the selectFirstPage param is set to TRUE and the current page in the loop is the first page, set it as the
		// currently active one. This is mainly used when calling the paging mechanics on page load or when the filters change.
		if(selectFirstPage && index == 0) {
			pageItem.addClass('currentPage');
			_events.hide();

			$.each(pages[pageNum - 1], function() {
				$('#' + this).show();
			});
		}
		
		pageItem.on('click', function() {
			_events.hide();
			
			$(this).addClass('currentPage');
			$(this).siblings().removeClass('currentPage');
			
			$.each(pages[pageNum - 1], function() {
				$('#' + this).show();
			});
		});
	});
}

// This function kicks off the generation of the necessary HTML to display the events
function RenderPoliticalEvents(fn, mobile,globalvarURL) {
	// The Event Location has to be queried seperately from the Events since the Location list and the Events list
	// exist on different webs (Sharepoint REST API limitation).
	GetPoliticalLocations(function(data) {
	    var locations = {};

	    $.each(data, function() {
	    	locations[this.ID] = this.Title;
	    });

		// The mobile param is used to show whether the script is being run by the mobile application or the
		// sharepoint site from an internet browser.
		if(mobile){
			GetPoliticalEventsMobile(function(data) {
				BuildEventObject(fn, data, locations);
			},globalvarURL);
		} else {
			GetPoliticalEvents(function(data) {
				BuildEventObject(fn, data, locations);
			});
		}
	},globalvarURL);
}

// This function kicks off the generation of the necessary HTML to display the events
function RenderCulturalEvents(fn) {
	// The Event Location has to be queried seperately from the Events since the Location list and the Events list
	// exist on different webs (Sharepoint REST API limitation).
	GetCultureMeetingLocations(function(data) {
	    var locations = {};

	    $.each(data, function() {
	    	locations[this.ID] = this.Title;
	    });
	    
		GetCulturalEvents(function(data) {
			BuildEventObject(fn, data, locations);
		});
	});
}

// This function converts the Events JSON objects into simpler JavaScript objects
function BuildEventObject(fn, data, locations) {
	var events = [];

    $.each(data, function() {
    	var _event = {
						ID: this.ID,
			    		Title: this.Title,
			    		EventDate: this.Event_x0020_Date,
			    		Location: locations[this.Political_x0020_LocationId],
			    		Type: this.Political_x0020_Event_x0020_TypeId,
			    		Topics: []
					};
					
		$.each(this.TopicId.results, function(index, value) {
    		var topic = {
    						ID: this.toString()
    					};

    		_event.Topics.push(topic);
    	});
    	
        events.push(_event);
        
    });
    
    //for put the divisor of old event
    var oldEventPlace = '<div class="blackback oldEvent"><div id="oldH" class="HeaderListFilt" align="center"><p class="p thick thick5">Past Events</p></div></div>';   
    var k = '';
    var firstold = false;
    
    events = SortEvents(events);
    
    $.each(events, function() {
        
        if(this.EventDate < k.EventDate )
        {
            $("#Event_placeholder").append(oldEventPlace+fn(this));    
        }
        else{
            $("#Event_placeholder").append(fn(this));    
        }
         
        k = this;
    });
	//fn(events);
}



function SortEvents(events) {
    var maxDate = moment(Math.max.apply(Math, events.map(function(o) { 
                    return moment(o.EventDate); 
    })));

    var currDate = moment();

    
    // If all dates are in the past, just return the array in the received order.
    if(maxDate.isSameOrAfter(currDate, 'day')) {
        var closestPreviousDate = moment(events[0].EventDate);
        var closestPreviousEventId = -1;
        $.each(events, function() {
            var currEventDate = moment(this.EventDate);
            if(currEventDate.isBefore(currDate, 'day')  && currDate.diff(currEventDate) <= currDate.diff(closestPreviousDate)) {
                closestPreviousDate = currEventDate;
                closestPreviousEventId = this.ID;
            }
        });

        var closestPreviousEventPos = events.map(function(o) {
           return o.ID;
        }).indexOf(closestPreviousEventId); 

        for(var i = 0; i <= closestPreviousEventPos; i++) {
            events.move(0, events.length - 1);
        }
    }

    return events;
}




// This function generates the necessary HTML to display the events
function RenderEvent(_event) {
	
	var strHtml = "<div class='col-xs-12 event-item' id='event-" + _event.ID + "'><div class='row row-no-gutter'><div class='col-xs-4 col-md-2 no-padding grid-col'><div class='row row-no-gutter'>";
	strHtml += "<div class='col-xs-6 no-padding'>" + moment(_event.EventDate).format('DD-MMM') + "</div><div class='col-xs-6 no-padding'>" + moment(_event.EventDate).format('HH:mm') + "</div></div></div>";
	strHtml += "<div class='col-xs-8 col-md-5 grid-col'>" + _event.Title + "</div><div class='col-xs-6 col-md-3 grid-col'>" + _event.Location + "</div><div class='col-xs-6 col-md-2 grid-col'>";
	
	var councils = [];
	
	$.each(_event.Topics, function() {
		var that = this;
		var councilTopics = $('.filter-item.council-item .topics .topic-item #topic-id').filter(function() { 
			return $(this).text() == that.ID 
		});
		
		var councilItems = councilTopics.closest('.council-item');
		$.each(councilItems, function() {
			var councilId = $(this).find('#council-id').html();
			var councilColour = $(this).find("[id$='colour']").html();
			var councilAcronym = $(this).find("[id$='acronym']").html();
			
			if(councils.indexOf(councilId) == -1) {
				councils.push(councilId);
				strHtml += "<div class='area-tag' style='background-color:" + councilColour + ";'>" + councilAcronym + "<div id='council-id'>" + councilId + "</div></div> ";
			}
		});
	});

	strHtml += "</div></div><div id='meeting-type-id'>" + _event.Type + "</div></div>";
	
	return strHtml;
}

function PopulateEventsContainer(events) {
	var eventsContainer = $('div.events-container .grid-content');
	var eventsGridGeader = $('div.events-container .grid-header');

	var currentEventTab = $('.filter-tabs .filter-tab.selected').attr('id');

	if(currentEventTab == 'political-tab') {
		eventsGridGeader.children('#area-col').html('POLICY AREA');
	} else {
		eventsGridGeader.children('#area-col').html('CULTURAL AREA');
	}

	eventsContainer.html('');

	$.each(events, function() {
		eventsContainer.append(RenderEvent(this));
	});
	
	PageEvents(true);
}

$(document).ready(function() {
	RenderPoliticalEvents(PopulateEventsContainer, false);
});